#*******************************************************************************
#****                           ShowBlocks                                  ****
#*******************************************************************************
import hou
def showBloks(blocks):
	for path in blocks:
		print "block:", 
		for coord in path:
			print ": ",
			x = float(coord[0])
			y = float(coord[1])
			print "(", x, ",", y, ")",
		print
           
def createGeometry(geoContainer, points, edges="", closed=False):
   geo = geoContainer.createNode("add")
   null = geoContainer.createNode("null", "HOM_Geo")

   geo.parm("points").set(len(points))
   geo.parm("prims").set(len(edges))
   
   for ptnum in range(len(points)):
      usept = "usept" + str(ptnum)
      pt = "pt"+str(ptnum)
      geo.parm(usept).set(1)
      geo.parm(pt+"x").set(points[ptnum][0])
      geo.parm(pt+"y").set(points[ptnum][1])
      geo.parm(pt+"z").set(points[ptnum][2])
      
      
   for edge in range(len(edges)):
      pr = "prim" + str(edge)
      geo.parm(pr).set(" ".join([str(x) for x in edges[edge]]))
      if closed:
         pr_closed = "closed"+str(edge)
         geo.parm(pr_closed).set(1)


   null.setFirstInput(geo)
   null.setHardLocked(1)
   geo.destroy()
         
   return null
      
# User interface:
def createEdges(geoContainer, points, edges):
   return createGeometry(geoContainer, points, edges)
   
def createFaces(geoContainer, points, poly):
   return createGeometry(geoContainer, points, poly, True)
   
def createPoints(geoContainer, points):
   return createGeometry(geoContainer, points)

def preprocess(blocks):
	print "entering..."
	polys = []
	verts = []
	currVert = 0
	print "going to iterate..."
	for path in blocks:
		print "B",
		face = [] 
		for (coordx,coordy) in path:
			print "(", coordx, ", ", coordy, ")",
			x = float(coordx)
			y = float(coordy)
			verts.append( (x,y,0) )
			face.append(currVert)
			currVert += 1
		print
		polys.append(face)
	return verts, polys

def blocks(blocks):
	container = hou.node("/obj").createNode("geo")
	for n in container.children(): 
		n.destroy()
	verts, polys = preprocess(blocks)
	print "going to create edges..."
	geometry = createFaces(container, verts, polys)

	print "going to reduce block sizes..."
	#reduce block size to leave space for the street
	poli = container.createNode('polyextrude')
	poli.setName('blockSize')
	poli.setFirstInput(geometry)
	poli.parm('lsx').set(0.8)
	poli.parm('lsy').set(0.8)
	poli.parm('outputside').set(0)

	poli.setDisplayFlag(1)

	for n in container.children(): 
		n.moveToGoodPosition()

def streets(streets):
	container = hou.node("/obj").createNode("geo")
	for n in container.children(): 
		n.destroy()
	verts, polys = preprocess(streets)
	print "going to create edges..."
	geometry = createEdges(container, verts, polys)

	for n in container.children(): 
		n.moveToGoodPosition()
