import random
import math

# ==========================
# ====    Class Vertex  ====
# ==========================
# Conte la imformacio de un vertex
# formada per els atributs seguents:
# cordenada x
# cordenada y
# jerarquia si es tracta de un carrer principal o secundari
# creixement informa sobre el nombre de arestes que van a parar al vertex
# id posicio dins la llista o estructura ordenada de vertex.
class Vertex:
  def __init__(self,x=0,y=0,hierarchy='',growing=0,id=-1):
    self.coord_x = x
    self.coord_y = y
    self.hierarchy = hierarchy      # jerarquia si es tracta de un carrer principal o secundari
    self.growing = growing          # creixement informa sobre el nombre de arestes que van a parar al vertex
    self.id = id                    # id posicio dins la llista o estructura ordenada de vertex.
    
  def getCoordX(self):
    return self.coord_x

  def getCoordY(self):
    return self.coord_y

  def getCoord(self):
    return (self.coord_x,self.coord_y)
    
  def setCoordX(self,x):
    self.coord_x = x   
        
  def setCoordY(self,y):
    self.coord_y = y

  def setCoord(self,x,y):
    self.coord_x = x
    self.coord_y = y
    
  def getHierarchy(self):
    return self.hierarchy
    
  def setHierarchy(self,hierarchy):
    self.hierarchy = hierarchy
    
  def getGrowing(self):
    return self.growing
    
  def setGrowing(self,growing):
    self.growing = growing     
       
  def getId(self):
    return self.id
    
  def setId(self,id):
    self.id = id    
    
  def printVertex(self):
    print 'x=%i y=%i grow=%i id=%i\n'%(self.coord_x,self.coord_y,self.growing,self.id)


# ==============================
# ====   Class VertexList   ==== 
# ==============================
# Conte una llista amb els nodes de la estructura de carrers ordenats per el seu id i sense repetits
# conte els atributs seguents:
# llista de vertex
class VertexList:
  def __init__(self):
    self.vertex_list = []

  def makeVertex(self,x=0,y=0,hierarchy='',growing=0):
    index = self.existVertex(x,y)        #comprovar repetits
    if index == -1:
      index = self.numVertex()
      vertex = Vertex(x,y,hierarchy,growing,index)
      self.vertex_list.append(vertex)
    #else:
    #  print 'Warning makeVertex try to insert existent vertex'
    return index
    
  def putVertex(self,vertex):
    index = self.existVertex(x,y)        #comprovar repetits
    if index == -1:    
      index = self.numVertex()
      vertex.setId(index)           #posa el id del vertex igual al index de la llista de vertex
      self.vertex_list.append(vertex)
    #else:
    #  print 'Warning putVertex try to insert existent vertex'
    return index
    
  def getVertex(self,index):
    return self.vertex_list[index]

  def getCoordVertex(self,index):
    return self.vertex_list[index].getCoord()
  
  def numVertex(self):
    return len(self.vertex_list)

  def existVertex(self,coord_x,coord_y):
    for vertex in self.vertex_list:
      if vertex.getCoordX() == coord_x and vertex.getCoordY() == coord_y:
        return vertex.getId()
    return -1 #no retorna false perque confon amb 0  

  def vertexProper(self, x, y, shappingmin, vdescartat=-1):
    vert_id = self.existVertex(x,y)
    if vert_id != -1:
      distancia_min = shappingmin
      for vertex in self.vertex_list:
        if vertex.getId() != vdescartat:
          distancia_act = math.sqrt(math.pow(vertex.getCoordX()-x,2)+math.pow(vertex.getCoordY()-y,2))
          if distancia_act <= distancia_min:
            distancia_min = distancia_act
            vert_id = vertex.getId()
    return vert_id

  def numVertexValencia(self, valencia):
    nombre = 0
    for vertex in self.vertex_list:
      if vertex.getGrowing() == valencia:
        nombre = nombre + 1
    return nombre    
  
  def printVertexList(self):
    for vertex in self.vertex_list:
      vertex.printVertex()

""" #no sutilitzen perque desindexen els vertex
  def removeVertexId(self,index):
    del self.vertex_list[index]
    #self.verex_list[index] = []
  def removeVertex(self,vertex):
    self.vertex_list.remove(vertex):
"""        


# ==================================
# ====    Class Street Class    ====
# ==================================
# Conte una llista de vertex que formen un conjunt d'arestes unides per els extrems o un carrer
# atributs:
# llista de ids de vertexs
# jerarquia informa sobre si es tracta de un carrer principal o secundari
class Street:
  def __init__(self,hierarchy=''):
    self.vertexes = []
    self.hierarchy = hierarchy    #informa sobre si es tracta de un carrer principal o secundari
  
  def getVertex(self,index):
    return self.vertexes[index]

  #inserta un nou vertex al final del carrer  
  def putVertex(self, vertex_id, vertex_list):
    index = self.numVertex()
    if(index != 0):
      #actualitza el numero d'arestes que passen pel vertex l'anomenat growing, el ultim carrer i el nou insertat a continuacio tenen un growing + 1
      vertexant_id = self.getVertex(index-1)
      growvant = vertex_list.getVertex(vertexant_id).getGrowing()
      grow = vertex_list.getVertex(vertex_id).getGrowing()    
      if grow >= 4 or growvant >= 4:
        #print 'Warning putVertex el growing es de 4'
        return False
      else:
        vertex_list.getVertex(vertexant_id).setGrowing(growvant+1)
        vertex_list.getVertex(vertex_id).setGrowing(grow+1)             
        self.vertexes.append(vertex_id)
        return True
    else:
      #es el unic vertex del carrer per tant no s'incrementa el growing
      self.vertexes.append(vertex_id)
      return True

  #inserta un nou vertex en la posicio determinada per el parametre index
  def insertVertex(self, index, vertex_id, vertex_list):
    numvertex = self.numVertex()
    growseg, growant = 0, 0
    if(numvertex != 0):
      #actualitza el numero de arestes que passen pel vertex o growing         
      if index == 0:     #si es vol insertar en la primera posicio del carrer
        vertexseg_id = self.getVertex(0)
        growseg = vertex_list.getVertex(vertexseg_id).getGrowing()
        if growseg < 4:
          vertex_list.getVertex(vertexseg_id).setGrowing(growseg+1)  
        growincrement = 1
      elif index == numvertex:       #si es vol insertar en la ultima posicio del carrer
        vertexant_id = self.getVertex(index - 1)
        growant = vertex_list.getVertex(vertexant_id).getGrowing()
        if growant < 4:
          vertex_list.getVertex(vertexant_id).setGrowing(growant+1)
        growincrement = 1       
      else:         #si es vol insertar en una posicio intermitja
        growincrement = 2
      grow = vertex_list.getVertex(vertex_id).getGrowing()
      if grow >= 4 or growseg >= 4 or growant >= 4:
        #print 'Warning insertVertex el growing es de 4'
        return False
      else:
        vertex_list.getVertex(vertex_id).setGrowing(grow+growincrement)
        self.vertexes.insert(index, vertex_id)
        return True
    else:
      #si es el unic vertex del carrer no s'incrementa el growing      
      self.vertexes.append(vertex_id)
      return True
      
  def numVertex(self):
    return len(self.vertexes)

  def existVertex(self, vertex_id):
    for index in range(self.numVertex()):
      if self.getVertex(index) == vertex_id:
        return index
    return -1      

  def getHierarchy(self):
    return self.hierarchy
    
  def setHierarchy(self,hierarchy):
    self.hierarchy = hierarchy
     
  def removeVertex(self,index):
    if index == self.numVertex or index == 0:
      print 'Error in removeVertex from street class growing alterat per no modificar un carrer intermig'
    del self.vertexes[index]
    
  def printStreet(self,vertex_list):
    for vertex_id in self.vertexes:
      vertex_list.getVertex(vertex_id).printVertex()     


# ==============================
# ====    Class StartNode   ====
# ==============================
class StartNode:
  def __init__(self,main_vertex,f=0.4,factor=1):
    self.node_list = [] ##cada element de la llista es una llista de 2 elements [id_vertex, probabilitat]
    self.main_vertex = main_vertex
    self.f=f
    self.factor=factor
    #self.interval=0
    
  def __probability(self,vertex):
    return math.exp(-self.f*math.sqrt(math.pow(self.main_vertex.getCoordX()-vertex.getCoordX(),2)+math.pow(self.main_vertex.getCoordY()-vertex.getCoordY(),2)))
       
  def __createList(self,vertexList):
    #criteri per si es vol espandir un node de valencia 1 o 2,3
    a = vertexList.numVertexValencia(2)
    b = vertexList.numVertexValencia(3)+ 2*vertexList.numVertexValencia(4)
    self.node_list = [] #es crea o borra la llista de nodes actuals
    if self.factor * b > a:
      #print 'Debug startNode nodes tipus 1 factor*b <= a %f*%f <= %f' %(self.factor,b,a)
      #contrueix una llista amb els nodes tipus 1
      for index in range(0,vertexList.numVertex()):
        vertex = vertexList.getVertex(index)
        if vertex.getGrowing() == 1:
          self.node_list.append([vertex.getId(),self.__probability(vertex)])
    else:
      #print 'Debug startNode nodes tipus 2,3 factor*b <= a %f*%f <= %f' %(self.factor,b,a)
      #construeix una llista dels nodes tipus 2 i 3
      for index in range(0,vertexList.numVertex()):
        vertex = vertexList.getVertex(index)
        if vertex.getGrowing() == 2 or vertex.getGrowing() == 3:
          self.node_list.append([vertex.getId(),self.__probability(vertex)])
          
    if self.node_list == []:      
      print 'Warnig llista buida factor*v=%i <= a=%i' %(self.factor * b,a)
      for index in range(0,vertexList.numVertex()):
        vertex = vertexList.getVertex(index)
        if vertex.getGrowing() < 4:
          self.node_list.append([vertex.getId(),self.__probability(vertex)])
     
  def __probabilityBubbleSort(self):
    for passesLeft in range(len(self.node_list)-1, 0, -1):
      for index in range(passesLeft):
        if self.node_list[index][1] < self.node_list[index + 1][1]:
          self.node_list[index], self.node_list[index + 1] = self.node_list[index + 1], self.node_list[index]
          
  def __brobabilityAcumulating(self):
    self.interval = 0
    for node in self.node_list:
      self.interval = self.interval + node[1]
      node[1] = self.interval
      
  def __randomNode(self):
    rand = random.uniform(0, self.interval)
    node_id = self.node_list[0][0]
    index = 0
    while self.node_list[index][1] <= rand and index<len(self.node_list):
        node_id = self.node_list[index+1][0]
        index = index + 1
    return node_id
  
  def initializeStartNode(self,vertexList):
    self.__createList(vertexList)
    self.__probabilityBubbleSort()
    self.__brobabilityAcumulating()
    #return self.__randomNode()
    
  def calculateProbabilisticStartNode(self):
    return self.__randomNode()

  def printStartNodeList(self):
    for node in self.node_list:
      print "id=%i probabilitat=%f\n" %(node[0],node[1])          

# =====================
# === obtenirIlles ====
# =====================
class city2Illes:
  def __init__(self,city):
    self.illes = []

    #crea una llista de arestes doble pero una en sentit contrari
    self.arestList = city.getArestList()
    arest2 = city.getArestList()
    for arest in arest2:
      self.arestList.append([arest[2], arest[3], arest[0], arest[1]])
   
  def __getAngle(self, vx2, vy2, vx1, vy1):
    if vx1 != vx2: #descarta el cas de la divisio per 0
        angle = math.atan(float(vy1 - vy2) / float(vx1 - vx2))
        if vx1 < vx2: #es mira la direccio va del primer punt al segon
          angle = math.radians(180) + angle
    else: #divisio per 0
      if vy1 > vy2:
        angle = math.radians(90)
      else:
        angle = math.radians(270)

    if angle < 0:
      angle = math.radians(360) + angle  
    return angle  

  #retorna una llista de posicions on es troben les arestes que comencen per el vertex a buscar
  def __findArest(self, x, y):
    posicions = []
    pos = 0
    for aresta in self.arestList:
      if aresta[0] == x and aresta[1] == y:
        posicions.append(pos)
      pos = pos + 1
    return posicions

  #busca entra les aresta seguents quina te l'angle mes petit
  def __angleMespetit(self, x1, y1, x2, y2):
    angle = angle = self.__getAngle(x2,y2,x1,y1)

    #obte la llista de arestes veines al segon vertex
    veines = self.__findArest(x2,y2)
    
    if len(veines) == 0:
      print 'Warning in getIslands no sha trobat vei per laresta %i %i (cas impossible de donar-se)' %(x2,y2)
      return {'recular': True, 'aresta_id': -1}
    #comprova si nomes tenim una aresta i es la mateixa duplicada pero a linversa    
    if len(veines) == 1:
      pos = veines[0]
      x3, y3 = self.arestList[pos][2], self.arestList[pos][3]
      if x3 == x1 and y3 == y1:
        return {'recular': True, 'aresta_id': pos}
      else:
        return {'recular': False, 'aresta_id': pos}

    angle_actual = 0
    pos = -1
    angle2, angle3 = 0, 0
    for veina in veines:
      x3, y3 = self.arestList[veina][2], self.arestList[veina][3]
      #print 'Debug vei %i %i %i %i' %(x3,y3,x1,y1)
      if x3 != x1 or y3 != y1: #comprova que no sigui la mateixa aresta pero duplicada a linversa
        angle2 = self.__getAngle(x2,y2,x3,y3)
        angle3 = angle - angle2
        if angle3 < 0:
          angle3 = math.radians(360) + angle3 
        #print 'Debug angle veina %i %f %f %f' %(veina,angle2,angle3,angle)
        if angle_actual < angle3:
          pos = veina
          angle_actual = angle3
    if pos == -1:
      print 'Error in getIslands no sha trobat vei2 per laresta %i %i %i %f %f %f(cas impossible de donar-se)' %(x2, y2, pos, angle2, angle3, angle_actual)
      print veines
      pos = veines[0]      
    return {'recular': False, 'aresta_id': pos}      

  #comprova si el vertex final de la illa esta repetit es a dir una illa completa
  def __illaTancada(self, illa):
    final = len(illa)
    (x, y) = illa[final-1]
    
    cicle_id = -1
    for pos in range(final-1):
      if illa[pos][0] == x and illa[pos][1] == y:
        cicle_id = pos
    return cicle_id    
      

  #retorna una llista de illes o barris
  def getIslands(self):
    #agafa la primera aresta de la llista
    illa = [(self.arestList[0][0],self.arestList[0][1]),(self.arestList[0][2],self.arestList[0][3])]
    del self.arestList[0]
    
    while self.arestList != []:    
      #comprova quina de les arestes veines te l'angle mes petit
      posicio = len(illa)
      (x1, y1) = illa[posicio-2]
      (x2, y2) = illa[posicio-1]
      seguent = self.__angleMespetit(x1, y1, x2, y2)

      #si nomes hi ha una aresta identica reculem la construccio de la illa un pas
      if seguent['recular'] == True:
        if seguent['aresta_id'] != -1:
          del self.arestList[seguent['aresta_id']]
        del(illa[posicio-1])  

      #expandeix la illa fins tancar-la
      else:
        x3, y3 = self.arestList[seguent['aresta_id']][2], self.arestList[seguent['aresta_id']][3]
        del self.arestList[seguent['aresta_id']]
        illa.append((x3,y3))
        completa = self.__illaTancada(illa)
        if completa != -1:
          if(len(illa) - completa > 3): #descarta illes de 1 o 2 vertex si ni a alguna (el primer i el ultim son el mateix)
            self.illes.append(illa[completa:len(illa)])
          illa = illa[0:completa+1]

      #si la illa actual no te cap aresta per continuar sagafa la seguent de la llista        
      if len(illa) <= 1 and len(self.arestList) > 0:
        illa = [(self.arestList[0][0],self.arestList[0][1]),(self.arestList[0][2],self.arestList[0][3])]
        del self.arestList[0]
          
      #print self.arestList
      #print '--------------'
      #print illa
      #print '----------'
      #print self.illes
      #print '\n'

    #borra la illa externa la mes gran
    mesgran = 0
    pos = 0
    borrar = -1
    #print self.illes    
    for island in self.illes:
      if len(island) > mesgran:
        mesgran = len(island)
        borrar = pos
      pos = pos+1  
    #print 'borrar la illa %i' %(borrar)
    if borrar != -1:
      del self.illes[borrar]
    
    return self.illes
  

# =========================
# ====    Class City   ====
# =========================
# Classe principal que conte tota la estructura de carrers
# atributs:
# llista de carrers
# una referencia a una classe vertex list
# 
class City:
  #parametres:
  #f          concentracio respecte el centre. Utilitzat alhora d'escollir el node de sortida de la expancio (com mes petit menys concentracio al centre)
  #angle_var  variacio del angle de sortida en graus al expandir un carrer (ex: -30 +30)
  #long_min   longitud minima d'un tram de carrer nou
  #long_max   longitud maxima d'un tram de carrer nou
  #allargament   alhora de calcular les interseccion es permet un allargament de x unitats
  #shappingmin   alhora de expandir un carrer si existeix un vertex a x unitats s'utilitza aquest en comptes de crear el nou vertex
  #factor     relacio entre els vertex amb valencia 2 i els de 4 factor utilitzat alhora d'escollir el node de sortida de la expancio
  #city_center_x, city_center_y   coordenades del centre de la ciutat. Utilitzat alhora d'escollir el node de sortida de la expancio
  def __init__(self, city_center_x=0, city_center_y=0, f=0.6, factor=0.4, angle_var=5, long_min=4, long_max=8, allargament=3, shappingmin=2):
    self.streets = []
    self.vertex_list = VertexList()
    city_center = Vertex(city_center_x, city_center_y)
    self.start_node = StartNode(city_center, f, factor)
    self.f = f
    self.factor = factor
    self.angle_var = angle_var
    self.long_min = long_min
    self.long_max = long_max
    self.allargament = allargament
    self.shappingmin = shappingmin

  def updateParameters(self, city_center_x=False, city_center_y=False, f=False, factor=False, angle_var=False, long_min=False, long_max=False, allargament=False, shappingmin=False):
    if city_center_x != False and city_center_y != False:
      city_center = Vertex(city_center_x, city_center_y)
      self.start_node = StartNode(city_center, f, factor)
    if f != False:  
      self.f = f
    if factor != False:
      self.factor = factor
    if angle_var != False:  
      self.angle_var = angle_var
    if long_min != False:  
      self.long_min = long_min
    if long_max != False:
      self.long_max = long_max
    if allargament != False:  
      self.allargament = allargament
    if shappingmin != False:  
      self.shappingmin = shappingmin
    
  def putStreet(self, street):
    self.streets.append(street)  
      
  def getStreet(self, index):
    return self.streets[index]
    
  def insertStreet(self, index, street):
    self.streets.insert(index,street)
    
  def numStreets(self):
    return len(self.streets)  

  def createStreet(self, street_vertexes, hierarchy=''):
    #crea un carrer inicial
    street = Street(hierarchy=hierarchy)
    for (x,y) in street_vertexes:
      #crea el vertex
      vt_id = self.vertex_list.makeVertex(x,y, hierarchy=hierarchy)
      street.putVertex(vt_id,self.vertex_list)
    self.putStreet(street)

  def createInitialCity(self, city_vertex, hierarchy=''):
    for street in city_vertex:
      self.createStreet(street, hierarchy)
  
  #donat un vertex_id retorna el index de carrer i la posicio del vertex on es troba el vertex
  def __getVertexStreetPosition(self, vertex_id):
    result = [] #retorn una llista de tuples [(streetpos,vertexpos),..]
    for streetIndex in range(self.numStreets()):
      find_vid = self.getStreet(streetIndex).existVertex(vertex_id)
      if find_vid != -1:
        result.append((streetIndex,find_vid))
    return result
 
  def __getAngle(self, vx2, vy2, vx1, vy1):
    if vx1 != vx2: #descarta el cas de la divisio per 0
        angle = math.atan(float(vy1 - vy2) / float(vx1 - vx2))
        if vx1 < vx2: #es mira la direccio va del primer punt al segon
          angle = math.radians(180) + angle
    else: #divisio per 0
      if vy1 > vy2:
        angle = math.radians(90)
      else:
        angle = math.radians(270)

    #if angle < 0:
    #  angle = math.radians(360) - angle  
    return angle      

  def __calculateNewVertex(self, x, y, angle, length):  #angle es en radiants
    coord_x = round(length * math.cos(angle)) + x
    coord_y = round(length * math.sin(angle)) + y
    return (coord_x, coord_y)
  
  #Donat un vertex_id intenta expandir-lo en la direccio mes correcta possible
  def __getNewVertexExpantion(self, vertex_id):   
    vertex = self.vertex_list.getVertex(vertex_id)

    if vertex.getGrowing() >= 4:
      print 'Warning getNewVertexExpantion s\'intenta expandir un carrer complet\n'
      return False
    
    positions = self.__getVertexStreetPosition(vertex_id) #obte una llista amb la posicio i carrer on es troba el vertex
    (vx, vy) = vertex.getCoord()

    #si el vertex te 3 o 1 arestes te que estar al principi o final d'un carrer
    if vertex.getGrowing() == 3 or vertex.getGrowing() == 1:
      for pos in positions:
        if pos[1] == 0: #es el principi o primer vertex del carrer
          vertseguent_id = self.getStreet(pos[0]).getVertex(1) #obte el vertex seguent
          vertseguent = self.vertex_list.getVertex(vertseguent_id)
          (vsx, vsy) = vertseguent.getCoord()
          
          #interpola un nou punt donat 2 punts
          angle_entrada = self.__getAngle(vsx,vsy,vx,vy)        
          angle_var_rad = math.radians(self.angle_var)
          angle_sortida = random.uniform(angle_entrada - angle_var_rad, angle_entrada + angle_var_rad)
          longitud = random.randint(self.long_min, self.long_max)
          (new_coord_x,new_coord_y) = self.__calculateNewVertex(vx, vy, angle_sortida, longitud)
          #print 'Expansion 1, %i %i, %i %i, %i %i' %(vsx,vsy,vx,vy,new_coord_x,new_coord_y)

          return {'x':new_coord_x, 'y':new_coord_y, 'expancio':1, 'street_id':pos[0], 'vertex_stpos': pos[1], 'angle':angle_sortida, 'longitud': longitud}

        elif pos[1] == (self.getStreet(pos[0]).numVertex() - 1): #es el vertex final del carrer
          vertanterior_id = self.getStreet(pos[0]).getVertex(pos[1] - 1) #obte el vertex_anterior
          vertanterior = self.vertex_list.getVertex(vertanterior_id)
          (vax, vay) = vertanterior.getCoord()

          #interpola un nou punt donat 2 punts
          angle_entrada = self.__getAngle(vax,vay,vx,vy)        
          angle_var_rad = math.radians(self.angle_var)
          angle_sortida = random.uniform(angle_entrada - angle_var_rad, angle_entrada + angle_var_rad)
          longitud = random.randint(self.long_min, self.long_max)
          (new_coord_x,new_coord_y) = self.__calculateNewVertex(vx, vy, angle_sortida, longitud)
          #print 'Expansion 2, %i %i, %i %i, %i %i' %(vax,vay,vx,vy,new_coord_x,new_coord_y)
          
          return {'x':new_coord_x, 'y':new_coord_y, 'expancio':2, 'street_id':pos[0], 'vertex_stpos': pos[1], 'angle':angle_sortida, 'longitud': longitud}

    #si el vertex te 2 arestes es tracta de un vertex intermig del carrer, podria fallar si focin 2 extrems de carrers diferents  
    elif vertex.getGrowing() == 2: 
      for pos in positions:
        if pos[1] > 0 and pos[1] < (self.getStreet(pos[0]).numVertex() - 1): 
          vertseguent_id = self.getStreet(pos[0]).getVertex(pos[1] + 1)
          vertseguent = self.vertex_list.getVertex(vertseguent_id)
          (vsx, vsy) = vertseguent.getCoord()
          vertanterior_id = self.getStreet(pos[0]).getVertex(pos[1] - 1)
          vertanterior = self.vertex_list.getVertex(vertanterior_id)
          (vax, vay) = vertanterior.getCoord()
          
          #interpola el nou vertex donat 3 punts i es fa un aleatori per triar direccio
          angle_entrada1 = self.__getAngle(vx,vy,vsx,vsy)
          angle_entrada2 = self.__getAngle(vx,vy,vax,vay)  
          angle_entrada = (angle_entrada1 + angle_entrada2) / 2.0
          if random.randint(0, 1) == 1: #aleatori si creem el nou carrer cap un costat o l'altre
            angle_entrada = angle_entrada + math.radians(180)
          angle_var_rad = math.radians(self.angle_var)
          angle_sortida = random.uniform(angle_entrada - angle_var_rad, angle_entrada + angle_var_rad)
          longitud = random.randint(self.long_min, self.long_max)
          (new_coord_x,new_coord_y) = self.__calculateNewVertex(vx, vy, angle_sortida, longitud)
          #print 'Expansion 3, %i %i, %i %i, %i %i, %i %i' %(vax,vay,vx,vy,vsx,vsy,new_coord_x,new_coord_y)         

          return {'x':new_coord_x, 'y':new_coord_y, 'expancio':3, 'street_id':pos[0], 'vertex_stpos': pos[1], 'angle':angle_sortida, 'longitud': longitud}
         
    #print 'Error streetExpantion no s\'ha trobat cap cas per xpandir en el vertex_id = %i growing = %i streets contenen = %i' %(vertex_id,vertex.getGrowing(),len(positions))
    # per no reiterar l'error es descarta el vertex com a completat
    self.vertex_list.getVertex(vertex_id).setGrowing(5) 
    return False

  #comproba si intersecten 2 fragments de rectes donats 4 punts
  def __intersect(self, xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2):
    #descarta interseccions segons quadrants rapid per mes eficiencia
    if max(xa1,xa2) < min (xb1,xb2) or min(xa1,xa2) > max(xb1,xb2) or max(ya1,ya2) < min(yb1,yb2) or min(ya1,ya2) > max(yb1,yb2):
      return False
    else:     
      #si es tracta de rectes o fregments identics no es calcula interseccio
      if (xa1 == xb1 and ya1 == yb1 and xa2 == xb2 and ya2 == yb2) or (xa1 == xb2 and ya1 == yb2 and xa2 == xb1 and ya2 == yb1):
        return False
       
      #equacio de la recta que passa per 2 punts
      #[y-y1]/ [x-x1] = [y2-y1]/ [x2-x1]

      #cas que una recta es horitzontal (evita divisio per 0 en la equacio generica)
      if xa1 == xa2:
          if xa1 == xa2 == xb2 == xb1:
            #retorna el vertex b1 o b2 de la interseccio perque ja estan creats cosa que el a2 sesta expandint        
            if xb1 >= max(min(xa1,xa2),min(xb1,xb2)) and xb1 <= min(max(xa1,xa2),max(xb1,xb2)) and yb1 >= max(min(ya1,ya2),min(yb1,yb2)) and yb1 <= min(max(ya1,ya2),max(yb1,yb2)):
              if xa1 == xb1 and ya1 == yb1:
                print "Warning intersect1 el mateix node que el de sortida %f %f %f %f, %f %f, %f %f, %f %f" %(x,y,xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)                                       
              return (xb1,yb1)
            elif xb2 >= max(min(xa1,xa2),min(xb1,xb2)) and xb2 <= min(max(xa1,xa2),max(xb1,xb2)) and yb2 >= max(min(ya1,ya2),min(yb1,yb2)) and yb2 <= min(max(ya1,ya2),max(yb1,yb2)):
              if xa1 == xb2 and ya1 == yb2:
                print "Warning intersect2 el mateix node que el de sortida %f %f %f %f, %f %f, %f %f, %f %f" %(x,y,xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)                           
              return (xb2,yb2)
            else:
              return False  
          x = xa1
          #print "Debug intersect rectes horitzontals %f %f, %f %f, %f %f, %f %f" %(xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
          y = ((yb2-yb1)/float(xb2-xb1))*(x-xb1)+yb1          
      elif xb1 == xb2:
          x = xb1
          y = ((ya2-ya1)/float(xa2-xa1))*(x-xa1)+ya1
          #print "Debug intersect rectes horitzontals2 %f %f, %f %f, %f %f, %f %f" %(xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
      else:
        #si tenen el mateix pendent son rectes paralleles
        if (ya2-ya1)/float(xa2-xa1) == (yb2-yb1)/float(xb2-xb1):
          #si les rectes estan una sobre l'altre es a dir la mateixa equacio de la recta pero fragments diferents
          pendent = (ya2-ya1)/float(xa2-xa1)
          if (-xa1*pendent + ya1) == (-xb1*pendent + yb1):
            #retorna el vertex b1 o b2 de la interseccio perque ja estan creats cosa que el a2 sesta expandint        
            if xb1 >= max(min(xa1,xa2),min(xb1,xb2)) and xb1 <= min(max(xa1,xa2),max(xb1,xb2)) and yb1 >= max(min(ya1,ya2),min(yb1,yb2)) and yb1 <= min(max(ya1,ya2),max(yb1,yb2)):
              if xa1 == xb1 and ya1 == yb1:
                print "Warning intersect3 el mateix node que el de sortida %f %f %f %f, %f %f, %f %f, %f %f" %(x,y,xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)                           
              return (xb1,yb1)
            elif xb2 >= max(min(xa1,xa2),min(xb1,xb2)) and xb2 <= min(max(xa1,xa2),max(xb1,xb2)) and yb2 >= max(min(ya1,ya2),min(yb1,yb2)) and yb2 <= min(max(ya1,ya2),max(yb1,yb2)):
              if xa1 == xb2 and ya1 == yb2:
                print "Warning intersect4 el mateix node que el de sortida %f %f %f %f, %f %f, %f %f, %f %f" %(x,y,xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)              
              return (xb2,yb2)
            else:
              return False            
          else:
            #print 'Debug __intersect paralleles que no creuen %f %f, %f %f, %f %f, %f %f' %(xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
            return False

        
        x = (yb1 - ya1 + ((ya2-ya1)/float(xa2-xa1))*xa1 - ((yb2-yb1)/float(xb2-xb1))*xb1) / (((ya2-ya1)/float(xa2-xa1)) - ((yb2-yb1)/float(xb2-xb1)))
        y = ((ya2-ya1)/float(xa2-xa1))*(x-xa1)+ya1
        #la mateixa equacio pero aquest cop aillant la incognita Y no funciona i obtenim mes divisions per 0
        #y = (xb1 - xa1 - yb1/((yb2-yb1)/float(xb2-xb1)) + ya1/((ya2-ya1)/float(xa2-xa1)))  * (((ya2-ya1)/float(xa2-xa1)) - ((yb2-yb1)/float(xb2-xb1)))
        #x = (y-ya1)/((ya2-ya1)/float(xa2-xa1)) + xa1        

      if xa1 == x and ya1 == y:
        #print "Warning intersect5 el mateix node que el de sortida %f %f, %f %f, %f %f, %f %f" %(xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
        return False

      #es mira que la interseccio trobada entri dintre dels 2 segments limitats i no rectes infinites
      if x >= max(min(xa1,xa2),min(xb1,xb2)) and x <= min(max(xa1,xa2),max(xb1,xb2)) and y >= max(min(ya1,ya2),min(yb1,yb2)) and y <= min(max(ya1,ya2),max(yb1,yb2)):
        return (round(x),round(y))
      else:
        return False

  #adapta el nou vertex creat en lexpancio en l'estructura de la ciutat, comprova interseccion, allargaments i reutilitza vertex existents
  def __adaptNewVertex(self, vertex_id, expantion):
    xa2, ya2 = expantion['x'], expantion['y']
    #return {'x':xa2, 'y':ya2, 'vertex_id': -1, 'street_id': -1, 'vertex_stpos': -1}
    #descartStreetId = expantion['street_id']           if streetId != descartStreetId: #descarta el carrer que conte el vertex original dexpancio
    angle = expantion['angle']
    longitud = expantion['longitud']
       
    vertex = self.vertex_list.getVertex(vertex_id)
    (xa1, ya1) = vertex.getCoord()    
    interseccions = []
    #comprova interseccions amb la resta de carrers
    streetId = 0
    for street in self.streets:
      for vertPos in range(street.numVertex()-1):
        vert1Id = street.getVertex(vertPos)
        (xb1, yb1) = self.vertex_list.getVertex(vert1Id).getCoord()
        vert2Id = street.getVertex(vertPos+1)
        (xb2, yb2) = self.vertex_list.getVertex(vert2Id).getCoord()
        if vert1Id != vertex_id and vert2Id != vertex_id:
          intersectPoint = self.__intersect(xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
          if intersectPoint != False:
            #print 'debug adapt interseccio1 %i %i %i %i %i %i %i %i %i %i' %(intersectPoint[0],intersectPoint[1],xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
            interseccions.append((intersectPoint[0],intersectPoint[1],streetId,vertPos))
      streetId = streetId + 1

    #si no s'han trobat interseccion
    #comprova interseccions amb allargament
    if len(interseccions) == 0:
      (xa3,ya3) = self.__calculateNewVertex(xa1, ya1, angle, longitud+self.allargament)
      streetId = 0
      for street in self.streets:
        for vertPos in range(street.numVertex()-1):
          vert1Id = street.getVertex(vertPos)
          (xb1, yb1) = self.vertex_list.getVertex(vert1Id).getCoord()
          vert2Id = street.getVertex(vertPos+1)
          (xb2, yb2) = self.vertex_list.getVertex(vert2Id).getCoord()
          if vert1Id != vertex_id and vert2Id != vertex_id:
            intersectPoint = self.__intersect(xa1, ya1, xa3, ya3, xb1, yb1, xb2, yb2)
            if intersectPoint != False:
              #print 'debug adapt interseccio2 %i %i %i %i %i %i %i %i %i %i' %(intersectPoint[0],intersectPoint[1],xa1, ya1, xa2, ya2, xb1, yb1, xb2, yb2)
              interseccions.append((intersectPoint[0],intersectPoint[1],streetId,vertPos))
        streetId = streetId + 1

    #si s'ha obtingut mes de una interseccio agafa la que esta mes propera al node d'expancio
    x, y, streetId, vertPos = xa2, ya2, -1, -1
    if len(interseccions) > 0:
      distancia_min = 10000000
      for intersect in interseccions:
        distancia_act = math.sqrt(math.pow(xa1-intersect[0],2)+math.pow(ya1-intersect[1],2))
        if distancia_act < distancia_min:
          distancia_min = distancia_act
          x, y, streetId, vertPos = intersect[0], intersect[1], intersect[2], intersect[3]

    #comprova el shapping o si existeix un vertex proper ja creat
    newvertex_id = self.vertex_list.vertexProper(x, y, self.shappingmin, vertex_id)

    return {'x':x, 'y':y, 'vertex_id': newvertex_id, 'street_id': streetId, 'vertex_stpos': vertPos}

  #crea el vertex si no existeix i reestructura la organitzacio dels carrers
  def __createNewVertexExpantion(self, vertex_id, expantion, adapt):
    #print 'Debug in createNewVertexExpantion'
    #print expantion
    #print adapt
    
    #comprova que el node d'expancio no sigui el mateix
    if adapt['vertex_id'] == vertex_id:
      print 'Warning in createNewVertexExpantion es vol expandir el mateix node que el de sortida'
      # per no reiterar l'error es descarta el vertex com a completat
      self.vertex_list.getVertex(vertex_id).setGrowing(5) 
      return False

    #si el vertex no existeix es crea un de nou
    if adapt['vertex_id'] == -1:
      newvertex_id = self.vertex_list.makeVertex(adapt['x'],adapt['y'])
    else:
      newvertex_id = adapt['vertex_id']

    #si s'ha trobat una interseccio es modifica el carrer afectat tambe
    if adapt['vertex_stpos'] != -1 and adapt['street_id'] != -1:
      estat3 = self.getStreet(adapt['street_id']).insertVertex(adapt['vertex_stpos'] + 1, newvertex_id, self.vertex_list)
      if estat3 == False:
        #print 'Warning in createNewVertexExpantion al retallar carrer per interseccio te el growing 4'
        # per no reiterar l'error es descarta el vertex com a completat
        self.vertex_list.getVertex(vertex_id).setGrowing(5) 
        return False

    #expandeix el carrer segons el tipus dexpancio executada
    if expantion['expancio'] == 1:
      estat = self.getStreet(expantion['street_id']).insertVertex(0, newvertex_id, self.vertex_list)
    elif expantion['expancio'] == 2:
      estat = self.getStreet(expantion['street_id']).insertVertex(expantion['vertex_stpos'] + 1, newvertex_id, self.vertex_list)
    elif expantion['expancio'] == 3:    
      new_street = Street()
      estat = new_street.putVertex(vertex_id, self.vertex_list)
      if estat == True:
        estat2 = new_street.putVertex(newvertex_id, self.vertex_list)
      if estat == True and estat2 == True:
        self.putStreet(new_street)
      else:
        #print 'Warning in createNewVertexExpantion expancio 2 te el growing 4'
        # per no reiterar l'error es descarta el vertex com a completat
        #vertex_list.getVertex(vertex_id).setGrowing(5)
        if adapt['vertex_stpos'] != -1 and adapt['street_id'] != -1:  #desfa el carrer modificat per la interseccio
          estat3 = self.getStreet(adapt['street_id']).removeVertex(adapt['vertex_stpos'] + 1)
        return False
    if estat == False:
      #print 'Warning in createNewVertexExpantion expancio valencia 1,3 te el growing 4'
      if adapt['vertex_stpos'] != -1 and adapt['street_id'] != -1:  #desfa el carrer modificat per la interseccio
        estat3 = self.getStreet(adapt['street_id']).removeVertex(adapt['vertex_stpos'] + 1)      
      return False
   
    return True  

  def makeExpation(self, n_expancions = 10, n_errors = 1000):
    cont_errors = 0
    cont_expancions = 0
    while cont_expancions < n_expancions and cont_errors < n_errors:
      self.start_node.initializeStartNode(self.vertex_list)
      node_id = self.start_node.calculateProbabilisticStartNode()
      
      #print "--LLISTA DE PROBABILITATS NODE SORTIDA--"
      #self.start_node.printStartNodeList()

      expantion = self.__getNewVertexExpantion(node_id)
      if expantion != False:
        adapt = self.__adaptNewVertex(node_id, expantion)
        create = self.__createNewVertexExpantion(node_id, expantion, adapt)
        if create == True:
          #print "--CARRERS DE LA CIUTAT--"
          #city.printCity()
          #print '--------------------'  
          cont_expancions = cont_expancions + 1
          #houdiniStreets.houBlocks(city.getCarrers())
          #input()

      if expantion == False or create == False:
          cont_errors = cont_errors + 1
          print '--------------------'

    #resultats
    print "Expanciones: %i, Warnings: %i" %(cont_expancions,cont_errors)          
    if cont_errors == n_errors:
      return False
    else:
      return True

  #retorna la llista d'arestes extreta de la estructura de carrers
  def getArestList(self):
    arestes = []
    for street in self.streets:
      for vertPos in range(street.numVertex()-1):
        vert1Id = street.getVertex(vertPos)
        (x1, y1) = self.vertex_list.getVertex(vert1Id).getCoord()
        vert2Id = street.getVertex(vertPos+1)
        (x2, y2) = self.vertex_list.getVertex(vert2Id).getCoord()
        arestes.append([int(x1), int(y1), int(x2), int(y2)])
    return arestes    

  def getCarrers(self):
    carrers = []
    for street in self.streets:
      carrer = []
      for vertPos in range(street.numVertex()):
        vertId = street.getVertex(vertPos)
        (x, y) = self.vertex_list.getVertex(vertId).getCoord()
        carrer.append((x,y))
      carrers.append(carrer)
    return carrers  

  def getIlles(self):
    illes = city2Illes(self)
    return illes.getIslands()
            
  def printCity(self):
    for street in self.streets:
      print 'Carrer\n'
      street.printStreet(self.vertex_list)
